function varargout = cipher(varargin)

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
                   'gui_Singleton',  gui_Singleton, ...
                   'gui_OpeningFcn', @cipher_OpeningFcn, ...
                   'gui_OutputFcn',  @cipher_OutputFcn, ...
                   'gui_LayoutFcn',  [] , ...
                   'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT

%%

% --- Executes just before cipher is made visible.
function cipher_OpeningFcn(hObject, eventdata, handles, varargin)

% Choose default command line output for cipher
handles.output = hObject;

%Set up defaults for our GUI data
handles.filename = '';
handles.cipherkey_filename = '';
handles.modified_filename = '';

% Update handles structure
guidata(hObject, handles);

%%

% --- Outputs from this function are returned to the command line.
function varargout = cipher_OutputFcn(hObject, eventdata, handles) 

% Get default command line output from handles structure
varargout{1} = handles.output;

%%

% --- Executes on button press in PUSHloadtext.
function PUSHloadtext_Callback(hObject, eventdata, handles)

%Retrieve the GUI data
h = guidata(gcf);

%Bring up a windows file browser
[filename, pathname] = uigetfile('*.txt', 'Choose file to encypt or decrypt');
if filename ~=0
    %Store the filename in the GUI data
    h.filename = [pathname,filename];
    guidata(gcf,h);
    
    %Update the edit box with the filename only
    set( h.EDITloadtext, 'string', filename );
end

%%

% --- Executes on button press in PUSHcipherkey.
function PUSHcipherkey_Callback(hObject, eventdata, handles)

%Retrieve the GUI data
h = guidata(gcf);

%Bring up a windows file browser
[filename, pathname] = uigetfile('*.m', 'Choose cipher key');
if filename ~=0
    %Store the cipher key filename in the GUI data
    h.cipherkey_filename = strrep([pathname,filename],'.m','');
    guidata(gcf,h);
    
    %Update the edit box with the cipher key filename
    set( h.EDITcipherkey, 'string', filename );
end

%%

% --- Executes on button press in PUSHopenfile.
function PUSHopenfile_Callback(hObject, eventdata, handles)

%Retrieve the GUI data
h = guidata(gcf);

%Open file using default windows program
if strcmp( h.filename,'') ~=1 
    winopen( h.filename );
end

%%

% --- Executes on button press in PUSHopenmodfile.
function PUSHopenmodfile_Callback(hObject, eventdata, handles)

%Retrieve the GUI data
h = guidata(gcf);

%Open last modified file using default windows program
if strcmp( h.modified_filename,'') ~=1 
    winopen( h.modified_filename );
end

%%

% --- Executes on button press in PUSHencrypt.
function PUSHencrypt_Callback(hObject, eventdata, handles)

%Retrieve the GUI data
h = guidata(gcf);

%Load the file selected and encrypt using selected cipher key
if ( strcmp( h.cipherkey_filename,'') ~=1 ) && ( strcmp( h.filename,'') ~=1 )
    
    %Start clock
    t0 = tic;
    
    %Load cipher key
    run( h.cipherkey_filename );
    
    %Encrypt file and write out to the same directory
    subcipher( 'encrypt', h.filename, cipher_key )
    h.modified_filename = strrep(h.filename,'.txt','-encrypt.txt');
    
    %Display message to info bar
    set(h.TEXTinfobar,'string',[h.filename,' encrpyted in ',num2str( toc(t0) ),'s.']);
    
    %Update GUI data
    guidata(gcf,h);
end

%%

% --- Executes on button press in PUSHdecrypt.
function PUSHdecrypt_Callback(hObject, eventdata, handles)

%Retrieve the GUI data
h = guidata(gcf);

%Load the file selected and encrypt using selected cipher key
if ( strcmp( h.cipherkey_filename,'') ~=1 ) && ( strcmp( h.filename,'') ~=1 )
    
    %Start clock
    t0 = tic;
    
    %Load cipher key
    run( h.cipherkey_filename );
    
    %Encrypt file and write out to the same directory
    subcipher( 'dencrypt', h.filename, cipher_key )
    h.modified_filename = strrep(h.filename,'.txt','-dencrypt.txt');
    
    %Display message to info bar
    set(h.TEXTinfobar,'string',[h.filename,' decrpyted in ',num2str( toc(t0) ),'s.']);
        
    %Update GUI data
    guidata(gcf,h);
end

%%

%subcipher
% A simple substitution cipher. In 'encrypt' mode, this function will
% replace particular string characters (i.e. letters of the alphabet plus
% any special ASCII characters) in a file containing the PLAINTEXT with an
% alternative sorting of the alphabet, the CIPHER KEY, which is provided.
% In the 'decrypt' mode, string characters of the ENCRYPTED message file
% will be replaced with their correct equivalent, thus decoding the
% message.
%
% LAST UPDATED by Andy French September 2012
%
% Syntax: subcipher( cipher_mode, filename, cipher_key )
%
% cipher_mode       'decrypt' or 'encrypt'
% filename          String containing location of file to encrypt or decrypt
% cipher_key        N x 2 cell aray. First column is the plaintext alphabet
%                   (or indeed a subset of it if not all characters are changed
%                   via encryption or decryption) and the second column is the
%                   encrypted alphabet. There must obviously be no repetitions
%                   as this is a simple single alphabet substitution cipher.

function subcipher( cipher_mode, filename, cipher_key )

%Display message to the command window
disp(' ')
disp('%%%%%%%%%%%%%% SUBCIPHER - by Andy French %%%%%%%%%%%%%% ')
disp(' ')
disp(['.... ',cipher_mode, ' of file ',filename,' commencing ...'] )

%Start clock
t0 = cputime;

%Open file filename (read only)
fid = fopen( filename, 'r' );

%Store filename text in a row vector A of characters
A = fscanf(fid, '%c');

%Close file
fclose(fid);

%Open file for writing, discarding any existing contents of an existing
%file of the same name
fid = fopen( strrep(filename,'.txt',['-',cipher_mode,'.txt']), 'w' );

%Step through cipher_key, replacing instances
%of the characters with their plaintext or enciphered equivalents
B = A;
dim = size(cipher_key);
if strcmp(cipher_mode,'encrypt')==1
    %Encrypt file contents
    for n=1:dim(1)
        indices = strfind( A, cipher_key{n,1} );
        B(indices) = cipher_key{n,2};
    end
else
    %Decrypt file contents
    for n=1:dim(1)
        indices = strfind( A, cipher_key{n,2} );
        B(indices) = cipher_key{n,1};
    end
end

%Write encrypted character array B to a appended
fwrite(fid, B );

%Close file
fclose(fid);

%Stop clock and print encryption time to the command window
disp( ['.... ',cipher_mode, ' of file ',filename,...
    ' completed in ',num2str(cputime - t0),' seconds.'] );

%End of code
