%textstats
% Prompt user for a .txt file. Loads text file and generates a frequency
% table of the letters within the file. This could be compared to a
% standard English source (e.g. Works of Shakspeare, Bible etc) for use in
% frequency analysis, which could help to break a substitution cipher.
%
% LAST UPDATED by A. French & J. Lee May 2015.

%Hard code the alphabet for which we will use for our frequency analysis
alphabet = ['AaBbCcDdEeFfGgHhIiJjKkLlMmNnOoPpQqRrSsTtUuVvWwXxYyZz!.,"?#@><;:()$%^&*_-{}[]~1234567890\/|+= ',char(10), char(13),];

% Create a cell array with each element being a letter from alphabet
alphabet_cell = cell(size(alphabet));
for n = 1:length(alphabet);
    alphabet_cell{n} = alphabet(n);
end

%Bring up a windows file browser
[filename, pathname] = uigetfile('*.txt', 'Choose text file to analyse');
if filename ~=0
    %Store the filename in the GUI data
    filename = [pathname,filename];
    
    %Load the text file
    fid = fopen( filename, 'r' );

    %Store text in a row vector t of characters
    t = fscanf(fid, '%c');
    
    %close the file
    fclose(fid); 
    
    %initialize a frequency table
    f=zeros(1,length(alphabet));
    
    %step through elements of alphabets and populate in f the number of
    %instances of occurances of alphabet characters in file 'filename'
    for n=1: length(alphabet)
        f(n)=length(strfind(t,alphabet(n)));
    end

    %scale frequency table such that total frequency = 100
    f=100*f/sum(f);
    
    %create and plot a bar chart
    figure('name','text stats', 'color', [1,1,1]);
    bar(1:length(alphabet),f);
    title(['textstats for file', filename]);
    set(gca, 'xtick',1:length(alphabet),'xticklabel', alphabet_cell, 'fontsize' , 10);
    axis tight
end

%End of code