%encryptor
% A GUI for a polyalphabetic cipher text file encryption system.
%
% By Winchester College ProgSoc. January 2015.

function varargout = encryptor(varargin)

% Begin initialization code - DO NOT EDIT
gui_Singleton = 1;
gui_State = struct('gui_Name',       mfilename, ...
    'gui_Singleton',  gui_Singleton, ...
    'gui_OpeningFcn', @encryptor_OpeningFcn, ...
    'gui_OutputFcn',  @encryptor_OutputFcn, ...
    'gui_LayoutFcn',  [] , ...
    'gui_Callback',   []);
if nargin && ischar(varargin{1})
    gui_State.gui_Callback = str2func(varargin{1});
end

if nargout
    [varargout{1:nargout}] = gui_mainfcn(gui_State, varargin{:});
else
    gui_mainfcn(gui_State, varargin{:});
end
% End initialization code - DO NOT EDIT

%%

% This code xecutes just before encryptor is made visible.
function encryptor_OpeningFcn(hObject, eventdata, handles, varargin)
global h
h = handles;

% Choose default command line output for encryptor
handles.output = hObject;
guidata(hObject, handles);

%Set up defaults for our GUI data
h.filename = '';
h.cipherkey_filename = '';
h.modified_filename = '';

%Set up a global variable which defines the run status of the encryptor.
%If encryptor is running, run_status = 1, otherwise 0.
h.run_status = 0;

%Initialise progress bar
set( h.TEXTprogress, 'visible', 'off' );

%Initialize info bar
set( h.TEXTinfobar,'string', 'Welcome to the amazing encryption machine!' );

%%

% Outputs from this function are returned to the command line.
function varargout = encryptor_OutputFcn(hObject, eventdata, handles)

% Get default command line output from handles structure
varargout{1} = handles.output;

%%

% Generate cipher key push button
function PUSHgeneratecipherkey_Callback(hObject, eventdata, handles)
global h

%Obtain the number of alphabets
n = str2num( get( h.EDITnumalphabets, 'string' ) );

%Generate cipher key text file
cipherkey = gen_cipherkey(n);

%Move Gencipherkey into the right directory
movefile(cipherkey, ['Cipher Keys\', cipherkey], 'f');

%Display feedback in the info bar
set( handles.TEXTinfobar, 'string', ['Cipher Key ', cipherkey, ' created.'] );

%%

% Executes on button press in PUSHloadtext.
function PUSHloadtext_Callback(hObject, eventdata, handles)
global h

%Bring up a windows file browser
[filename, pathname] = uigetfile('*.txt', 'Choose file to encypt or decrypt');
if filename ~=0
    %Store the filename in the GUI data
    h.filename = [pathname,filename];
    
    %Update the edit box with the filename only
    set( h.EDITloadtext, 'string', filename );
end

%%

% Executes on button press in PUSHcipherkey.
function PUSHcipherkey_Callback(hObject, eventdata, handles)
global h

%Bring up a windows file browser
[filename, pathname] = uigetfile('*.txt', 'Choose cipher key');
if filename ~=0
    %Store the cipher key filename in the GUI data
    h.cipherkey_filename = strrep([pathname,filename],'.m','');
    
    %Update the edit box with the cipher key filename
    set( h.EDITcipherkey, 'string', filename );
end

%%

% Executes on button press in PUSHopenfile.
function PUSHopenfile_Callback(hObject, eventdata, handles)
global h

%Open file using default windows program
if strcmp( h.filename,'') ~=1
    winopen( h.filename );
end

%%

% Executes on button press in PUSHopenmodfile.
function PUSHopenmodfile_Callback(hObject, eventdata, handles)
global h

%Open last modified file using default windows program
if strcmp( h.modified_filename,'') ~=1
    winopen( h.modified_filename );
end

%%

% Executes on button press in PUSHencrypt.
function PUSHencrypt_Callback(hObject, eventdata, handles)
global h

%Load the file selected and encrypt using selected cipher key
if ( strcmp( h.cipherkey_filename,'') ~=1 ) && ( strcmp( h.filename,'') ~=1 )
    
    %Start clock
    t0 = tic;
    
    %Enable progress bar
    set( h.TEXTprogress, 'visible', 'on' );
    
    %Display message to info bar
    set(h.TEXTinfobar,'string',[h.filename,' encrpyting ...']);
    
    %Encrypt file and write out to the same directory
    output_filename = polycipher( h.filename, h.cipherkey_filename, 'encrypt' );
    h.modified_filename = output_filename;
    
    %Disable progress bar
    set( h.TEXTprogress, 'visible', 'off' );
end

%%

% Executes on button press in PUSHdecrypt.
function PUSHdecrypt_Callback(hObject, eventdata, handles)
global h

%Load the file selected and encrypt using selected cipher key
if ( strcmp( h.cipherkey_filename,'') ~=1 ) && ( strcmp( h.filename,'') ~=1 )
    
    %Disable progress bar
    set( h.TEXTprogress, 'visible', 'on' );
    
    %Display message to info bar
    set(h.TEXTinfobar,'string',[h.filename,' decrpyting ...']);
    
    %Encrypt file and write out to the same directory
    output_filename = polycipher( h.filename, h.cipherkey_filename, 'de-encrypt' );
    h.modified_filename = output_filename;
    
    %Disable progress bar
    set( h.TEXTprogress, 'visible', 'off' );
end

%%

%  --- Executes on button press in EDITnumalphabets
function EDITnumalphabets_Callback(hObject, eventdata, handles)
global h

%Make sure no invalid inputs are entered
if isempty( str2num( get( h.EDITnumalphabets,'string' ) ) )
    set( h.EDITnumalphabets, 'string', '3' );
end

%%

% Executes on button press in PUSHstop
function PUSHstop_Callback(hObject, eventdata, handles)
global h;

%Display a stopping message to the info bar
set( h.TEXTinfobar, 'string', ['Encryptor stopped! Error code ',num2str( randi(1e13,1) ),'. Illegal bust following dribble over worms.'] );

%Change run_status to 0
h.run_status = 0;

%%

%polycipher
% Encrypts or decrypts a text file based upon another text file which defines
% the cipher key.
%
% Syntax:
%
% filename - UNC path to the .txt file being encrypted or decrypted
% key_filename - A text file which contains the cipher key. The first line
% is the plaintext alphabet. The second line (which must be the same
% length) contains a permutation of the alphabet, and this represents the
% encrpted alphabet.
% N is the number of shuffled alphabets
% encrypt - If this is 'encrypt' then encrypt the file. Anything else,
% decrypt.
%
% filename = 'plaintext.txt'
% key_filename = 'cipher_key.txt'
% encrypt = 'encrypt'
% polycipher( filename, key_filename, N, encrypt )

function output_filename = polycipher( filename, key_filename, encrypt )

%% Max characters per iteration %%
max_chars_per_iteration = 1e5;

%

%Allow this to see the current run status, and other GUI data in h
global h;

%Set run status to be 1
h.run_status = 1;

%Start clock
t0=tic;

%Determine encryption mode
if strcmp( encrypt,'encrypt')==0
    mode = 'decrypt';
else
    mode = 'encrypt';
end

%Locate the character of last \ in filename
i = strfind( filename,'\' );
last_slash = i(end);

%Construct output filename
output_filename = [ filename( 1 : last_slash ),'-',...
    mode,'-',filename( last_slash + 1 : end ) ];

%Open file filename (read only)
fid = fopen( filename, 'r' );

%Store filename text in a row vector A of characters
A = fscanf(fid, '%c');

%Close file
fclose(fid);

%Load our cipher key
fid = fopen( key_filename, 'r' );

%Store cipher key text in a row vector K of characters
K = fscanf(fid, '%c');

%Determine the number of alphabets from the first two digits of the ciopher
%key character array K
N = str2num( K(1:2) );

%Strip off first two digits
K = K(3:end);

%Determine the length of each alphabet
L = length(K)/(N+1);

%Close file
fclose(fid);

%First half is the plaintext alphabet, second half is the shuffled alphabet
plaintext_alphabet = K( 1:L);
start = 1;
fin = L;
for n=1:N
    start = fin+1;
    fin = start + L-1;
    shuffled_alphabet{n} = K( start : fin );
end

%Work out number of iterations for possible status bar update
M = ceil( length(A)/max_chars_per_iteration );

%Initialize the alphabet to use
a = 1;

%Step through A, and exchange characters for their characters in the cipher
%key plaintext alphabet. Output vector is B.
B = A;
c = 1;

%Select mode and encrypt or decrypt
switch mode
    case 'decrypt'
        for m=1:M
            for n=1:max_chars_per_iteration
                if c <= length(A)
                    %Find the index of shuffled alphabet which corresponds to A(n)
                    i = strfind( shuffled_alphabet{a}, A(c) );
                    
                    %Set character in output vector B to be the corresponding letter from
                    %plaintext alphabet
                    if isempty(i)==0
                        B(c) = plaintext_alphabet(i);
                    end
                    
                    %Advance a by one, and make sure it isn't larger than the amount of
                    %alphabet keys available
                    a = a + 1;
                    if a > N
                        a = 1;
                    end
                    
                    %Increment counter
                    c = c + 1;
                else
                    break;
                end
            end
            
            %Check if stop button has been pressed
            if h.run_status == 0
                %Display a stopping message to the info bar
                set( h.TEXTinfobar, 'string', ['Encryptor stopped! Error code ',...
                    num2str( randi(1e13,1) ),'. Illegal bust following dribble over worms.'] );
                return;
            end
            
            %Update status bar
            set( h.TEXTprogress, 'position', [15,16,660*m/M,25] );
            drawnow;
        end
    case 'encrypt'    
        for m=1:M
            for n=1:max_chars_per_iteration
                if c <= length(A)
                    %Find the index of plaintext alphabet which corresponds to A(n)
                    i = strfind( plaintext_alphabet, A(c) );
                    
                    %Set character in output vector B to be the corresponding letter from
                    %shuffled alphabet
                    if isempty(i)==0
                        B(c) = shuffled_alphabet{a}(i);
                    end
                    
                    %Advance a by one, and make sure it isn't larger than the amount of
                    %alphabet keys available
                    a = a + 1;
                    if a > N
                        a = 1;
                    end
                    
                    %Increment counter
                    c = c + 1;
                else
                    break;
                end
            end
            
            %Check if stop button has been pressed
            if h.run_status == 0
                %Display a stopping message to the info bar
                set( h.TEXTinfobar, 'string', ['Encryptor stopped! Error code ',...
                    num2str( randi(1e13,1) ),'. Illegal bust following dribble over worms.'] );
                return;
            end
            
            %Update status bar
            set( h.TEXTprogress, 'position', [15,16,660*m/M,25] );
            drawnow;
        end
end

% Write out the encrypted or de-crypted file, unless stop button has been
% pressed
if h.run_status == 1
    
    %Open file for writing, discarding any existing contents of an existing
    %file of the same name
    fid = fopen( output_filename, 'w' );
    
    %Write encrypted character array B to a appended
    fwrite(fid, B );
    
    %Close file
    fclose(fid);
    
    %Display message to info bar
    set(h.TEXTinfobar,'string',[filename,' processed in ',num2str( toc(t0) ),'s.']);
    
    %Set run status to be 0
    h.run_status = 0;
end

%%

%gen_cipherkey
% Generate our cipher key and write out this in a file. This will be a
% set of N random alphabets to enable a polyalphabetic cipher to be
% encrypted. Note the first two characters of the output filename define
% the number of alphabets (N) (Excluding the original).

function cipherkey = gen_cipherkey(N)

%Define the original alphabet
alphabet = ['AaBbCcDdEeFfGgHhIiJjKkLlMmNnOoPpQqRrSsTtUuVvWwXxYyZz!.,"?#@><;:()$%^&*_-{}[]~1234567890\/|+= ',char(13),char(15)];

%%

%Work out N as a two character string
N = round(N);
if N<1
    N_str = '01';
elseif N<10
    N_str = ['0',num2str(N)];
elseif N<=99
    N_str = num2str(N);
else
    N_str = '01';
end

%Define an output array
A = [N_str,alphabet];

for n=1:N
    %Shuffle alphabet randomly
    L = length(alphabet);
    shuffled_alphabet = char(L);
    s = randperm(L);
    for m=1:L
        shuffled_alphabet(m) = alphabet( s(m) );
    end
    A = [A,shuffled_alphabet];
end

%Define the cipher key filename based on the current date and time
date_string = strrep( datestr(now), ':','-' );

%Open a text file and write the original alphabet and then the shuffled
%alphabet
cipherkey = ['cipher_key ',date_string,'.txt'];
fid = fopen( cipherkey, 'w' );
fwrite( fid, A );

%Close the file
fclose(fid);

%End of code
